/*
 * @author John Coffelt
 * @author Matthew Robertson
 */

precision mediump float;
 
uniform vec3 uLightDirection;
uniform vec3 uCameraPosition;
uniform sampler2D uTexture;
 
varying vec2 vTexcoords;
varying vec3 vWorldNormal;
varying vec3 vWorldPosition;
 
const vec3 lightColor = vec3(1.0, 1.0, 1.0);// #7
const vec3 materialColor = vec3(0.3, 0.3, 0.3);// #8
 
void main(void) {
    // vec4 check;
 
    vec3 albedo = texture2D(uTexture, vTexcoords).rgb;
    vec3 ambient = albedo * 0.1;
 
 
    //--- diffuse contribution ----------------------------------------------//
    /*
   Todo #1 light direction -- normalize & store in a separate variable
   uLightDirection is a 3 component vector uniform
   a normalized version is needed to compute the lambert term
   (don't assume it came in normalized)
 
   visualize: set the RGB values gl_FragColor to uLightDirection
        use `1.0` for alpha value “A”
   expect: https://lh3.googleusercontent.com/a_pezxAS24iu6CbWtWK1wqkk2pSFKs2pghEJNKDqz4huUqcAuqAzykN5wKfC6uqCL0pBh0hKMQoGnAnAqPc9CTCyzdIAKis07NTqN6T1rDhb11Nh7WXDi_8oOxzLt_CDnQ
 
   store a normalized copy & visualize it
   expect: https://lh4.googleusercontent.com/Yloi5sKiaYco-xfVWNmv9p1zJMmohuzdA03DKPPrz6aq3bKnATkuWynzhSedGqPnXx_J7tK5At0FklmQ6svVi-4PFALBSpELUOmMiSTAfzuGGkdrV9ZeHyE9LzgUqeDOQw
   */
    //    check = vec4(uLightDirection, 1.0);
 
    vec3 nLightDirection = normalize(uLightDirection);
    //    check = vec4(nLightDirection, 1.0);
 
    /*
    Todo #2 world space normal -- normalize & store in a separate variable
    vWorldNormal came from the vertex shader
    it is varying -- its value is interpolated for every pixel
 
    store a normalized copy & visualize it
    expect: https://lh3.googleusercontent.com/SGYPvmTAeC1WZ65Ugeex9on2qiivpT7mZOGDskLKYtzLrLeNphPGoJBNoPxO_MzLLOPAZJiJJaUMfGKVvYK33yPIch2feAGOJLAG5h10StilVATvnvvsjExAQn94Zg3mDg
    notes:
        red is right, green is up & blue is bottom left.
        since the sphere was not translated or rotated, it’s local space appears to match world space (no, it is scaled)
    */
    vec3 nWorldNormal = normalize(vWorldNormal);
    //    check = vec4(nWorldNormal, 1.0);
 
    /*
    Todo #3 Calculate the lambert term
    calculate the lambert term using the normalized light direction & world space normal
    visualize it
    notes:
        don’t worry about material & light color, visualize only light attenuation
        gl_FragColor is a vec4
        set RGB values to all be the same value to get the grayscale result
    expect: https://lh3.googleusercontent.com/UpOifM9NXd0AZmsv4aB0UaBFzLXyURKzkBqb_VZyxjWAPo916clNr3ylHV2_P6GwAmjK16cCSKH-U9QFN4Gl3dq3Sz4JF12xzIQgvikgXIdEnw-HyWWRtLp9gVG9esiXlg
    */
    float lambert = max(dot(nLightDirection, nWorldNormal), 0.0);
    //    check = vec4(lambert, lambert, lambert, 1.0);
 
 
    //--- specular contribution ---------------------------------------------//
    /*
    Todo #4 Calculate & store the eye vector
    in world space, calculate the direction from the surface point to the eye (normalized)
    it should start from the world space position and go toward the eye (normalized).
    expect: https://lh3.googleusercontent.com/qu_cRNoINEdlpnxeMa2KsjGft32Ui581IRtfkDXBzLLWsCiGi5Q7_2FoaQ7klbm-WYjoJoS6JDNggvDIeNBQ8lLd202jwi4WniPaKCi5e53b5jysg5FrY0BH4E5FWrCG9A
    */
    vec3 nEyeVector = normalize(uCameraPosition - vWorldPosition);
    //    check = vec4(nEyeVector, 1.0);
 
    /*
    Todo #5 Calculate & store reflection vector
    in world space, calculate the reflection vector (normalized)
 
 
    you can now calculate the light reflection with the eye vector & reflection vector
    compute & visualize the light reflections
    R = 2(N⋅L)N−L.
    expect: https://lh3.googleusercontent.com/TFRiywNbMRkpEjFS7qDm7VqTBvC72AlcELjBao9U3FqboQcXUk-GcPIi8bHzWAtNmQ_nb0zYPaRFD39eEdEdd36gjY96DyovVmMZU4KztaBQDb9VpEDCB3ju-n5NuG2eBw
 
    */
    vec3 reflectionVector = 2.0 * dot(nWorldNormal, nLightDirection) * nWorldNormal - nLightDirection;
    //    check = vec4(reflectionVector, 1.0);
 
    /*
    Todo #6 Calculate the Phong term
    the vectors to compute the Phong term
    calculate the phong term using a specular power (n) of `64`
    Specular ≈ (V⋅R)^n.
 
    notes:
        ignore the light & material parts (for now)
        calculation result will be scalar
 
    set each of RGB to be your computed scalar value
 
    expect: https://lh5.googleusercontent.com/AxjK4vxqe8gjUQ4whIwTFFAFfxDsfidGCrXywsFjTxt1vep2VhvFmUA5M6zdPlkM05x8z6CwKGMZg1EdLqi0FYbfrrtv98AlvhM-eJN05KbDUGn97xhdohTVXd3MONZDKQ
    (default camera angle)
    expect: https://lh3.googleusercontent.com/EeO7uISAXVYsUO1eEe4z2xQNTljdIvmvhqylu-ojiE3UUMlkNqbXKg8yjUnPVvkJDuy0N7p2vXK-k4lF_TjefK5P-ArlADsKOJbrhR_wH2KlhOzo1HAzBemu_PCAkddZEQ
    (rotated camera)
    */
 
    float phong = pow(max(dot(reflectionVector, nEyeVector), 0.0), 64.0);
   // check = vec4(phong, phong, phong, 1.0);
 
    //--- combine -----------------------------------------------------------//
    /*
    Todo #7 Combine light & material color for diffuse contribution
    1. apply light and material interaction for diffuse value by using the texture color as the material
 
    using the texture color as the material color, attenuate it via the lambert term and visualize the result.
    assume the light color is (1, 1, 1) which should not affect the final result
    expect: https://lh4.googleusercontent.com/4rOIdBTocWyRCC0OdWtSkxtrBG0hhLiIJYPKW1KDcizdf_sMc5Fym13pm4TN-arcXlquZT57NwV5YucUraStBaFXtyIycSu7HPP0dydyjqkyweQDiw8GMEj2bgmRqR5n3g
    */
    vec3 diffuseColor = albedo * lightColor * lambert;
    //    check = vec4(diffuseColor, 1.0);
 
    /*
    Todo #8 Combine light and material color for the specular contribution
    2. apply light and material interaction for phong, assume phong material color is (0.3, 0.3, 0.3)
    use RGB = (0.3, 0.3, 0.3) as your specular material color and white as your light color as before.
    expect: https://lh3.googleusercontent.com/29KIabDuXWO0JTDPpM5S_xekRufCNBCCNHPJ95E3u66XMS15c0_ddB0Q7OSjE14fussWmDxX9yyjAqgnyR5O0RVywMhigeZaPPeb5fJsYX6uzYU1bq0nWGSOuwJG44A4bg
 
    notes:
    the contribution is pretty weak with directional light, stronger with point light
    for a more noticable result, increase the specular material color (RGB) value
 
    */
 
    vec3 specularColor = materialColor * lightColor * phong;
    //check = vec4(specularColor, 1.0);
    /*
    Todo #9 Combine ambient, diffuse, and specular for final result
    see: https://lh4.googleusercontent.com/MKEjCls5D6RB64QLxbHHzbapXl0Gl0xl7L6leDdZNFU922ZnXsKyon5hTkzsGz16Fz3S7cpqCtSo76qFcE_8UrBKsE3XQBiIO5yf7TAY4yQlMnRespUqpLUYj_4nEk2KQA
    */
    vec3 finalColor = ambient + diffuseColor + specularColor;
 
    //    gl_FragColor = check;
    gl_FragColor = vec4(finalColor, 1.0);
}